/* BOGL - Ben's Own Graphics Library.
   Written by Ben Pfaff <pfaffben@debian.org>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "bogl.h"

extern struct bogl_font font_helvB10;
extern struct bogl_font font_helvB12;
extern struct bogl_font font_helvR10;
extern struct bogl_font font_timBI18;

extern struct bogl_pointer pointer_arrow;

struct bogl_font *font_list[] =
  {
    &font_helvB10,
    &font_helvB12,
    &font_helvR10,
    &font_timBI18,

    NULL
  };

extern struct bogl_pixmap pixmap_tux75;

struct bogl_pixmap *pixmap_list[] =
  {
    &pixmap_tux75,
    NULL
  };

#define n_pixmaps ((int) (sizeof (pixmap_list)			\
			  / sizeof (struct bogl_pixmap *)) - 1)

/* Print a helpful syntax message and terminate. */
void
usage (void)
{
  struct bogl_font **font;

  printf ("usage:\t\"bogl-test test-type\"\n"
	  "\twhere test-type may be one of the following:\n\n"
	  "\thline\thorizontal lines\n"
	  "\tvline\tvertical lines\n"
	  "\tbox\tsolid boxes\n"
	  "\ttext\ttext drawing (specify font from list below)\n"
	  "\tpointer\ttest pointer drawing & erasing\n"
	  "\tput\tpixmap drawing (specify pixmap from 0 to %d)\n\n"
	  "available fonts:\n",
	  n_pixmaps - 1);

  for (font = font_list; *font; font++)
    printf ("\t%s\n", (*font)->name);

  
  exit (0);
}

int
main (int argc, char *argv[])
{
  static struct bogl_cliprect fred[2]={
  	{10,10, 40 ,40},
  	{100,100,300,300}
  };
  
  if (argc < 2)
    usage ();

  if (!bogl_init ())
    {
      printf ("bogl: %s\n", bogl_error ());
      return 0;
    }
    
    bogl_clipinit(2,fred);

  if (!strcmp (argv[1], "hline"))
    for (;;)
      {
	int x1, x2, t;

	x1 = rand () % bogl_xres;
	x2 = rand () % bogl_xres;
	if (x1 > x2)
	  t = x1, x1 = x2, x2 = t;
	bogl_hline (x1, x2, rand () % bogl_yres, rand () % bogl_ncols);
      }

  if (!strcmp (argv[1], "vline"))
    for (;;)
      {
	int y1, y2, t;

	y1 = rand () % bogl_yres;
	y2 = rand () % bogl_yres;
	if (y1 > y2)
	  t = y1, y1 = y2, y2 = t;
	bogl_vline (rand () % bogl_xres, y1, y2, rand () % bogl_ncols);
      }

  if (!strcmp (argv[1], "box"))
    for (;;)
      {
	int x1, x2, y1, y2, t;
	x1 = rand () % bogl_xres;
	x2 = rand () % bogl_xres;
	if (x1 > x2)
	  t = x1, x1 = x2, x2 = t;
	y1 = rand () % bogl_yres;
	y2 = rand () % bogl_yres;
	if (y1 > y2)
	  t = y1, y1 = y2, y2 = t;
	bogl_clear (x1, y1, x2, y2, rand () % bogl_ncols);
      }

  if (!strcmp (argv[1], "text"))
    {
      struct bogl_font **font;

      if (argc < 3)
	usage ();

      for (font = font_list; *font; font++)
	if (!strcmp ((*font)->name, argv[2]))
	  break;
      if (!font)
	usage ();
      
      for (;;)
	{
	  char s[80];
	  int len;
	  int fg, bg;
	  int i;
      
	  fg = rand () % bogl_ncols;
	  bg = rand () % bogl_ncols;
	  if (fg == bg)
	    continue;
      
	  len = rand () % 64;
	  for (i = 0; i < len; i++)
	    s[i] = rand () % (126 - 32) + 32;
	  s[len] = 0;

	  bogl_text (rand () % bogl_xres, rand () % bogl_yres, s, len,
		     fg, bg, *font);
	}
    }
  
  if (!strcmp (argv[1], "put"))
    {
      struct bogl_pixmap *pixmap;
      int index;
      int color_map[] = {6, 7, 8, 9, 10, 11, 12, 13};
      
      if (argc < 3 || (index = atoi (argv[2])) < 0 || index >= n_pixmaps)
	usage ();
      pixmap = pixmap_list[index];
      
      bogl_set_palette (6, 8, pixmap->palette);

      bogl_refresh = 1;
      for (;;)
	{
	  if (bogl_refresh)
	    {
	      bogl_refresh = 0;
	      bogl_clear (0, 0, bogl_xres, bogl_yres, 1);
	      bogl_put (0, 0, pixmap, color_map);
	    }
	  pause ();
	}
      
      exit (0);
    }

  if (!strcmp (argv[1], "pointer"))
    {
      bogl_clear (0, 0, bogl_xres, bogl_yres, 1);

      for (;;)
	{
	  int x = rand () % bogl_xres;
	  int y = rand () % bogl_yres;
	  
	  bogl_pointer (1, x, y, &pointer_arrow, (int []) {15, 0});
	  getchar ();

	  bogl_pointer (0, x, y, &pointer_arrow, NULL);
	  getchar ();
	}
    }
  
  usage ();
}
